local assets=
{
    Asset("ATLAS", "images/inventoryimages/glutto_soul.xml"),
    Asset("ANIM", "anim/glutto_soul.zip"),
}

local SPAWN_DIST = 30
local ownershiptag = 'uid_private'


--------------------------------------------------------------
local function SpawnGlutto_new(inst)
    if inst ~= nil and inst.components.petleash ~= nil and not inst.components.petleash:IsFull() then
        inst.nospawnfx = true
        inst.components.petleash:SpawnPetAt(inst.Transform:GetWorldPosition())
		SpawnPrefab("small_puff").Transform:SetPosition(inst.Transform:GetWorldPosition())
        inst.nospawnfx = nil
    end
end

local function StartSpawn(inst)
	inst.AnimState:PlayAnimation("idle", true)
    inst:ListenForEvent("animover", SpawnGlutto_new)
end

local function OnDropped(inst)
    StartSpawn(inst)
end

local function OnSpawnFX(pet)
    SpawnPrefab("small_puff").Transform:SetPosition(pet.Transform:GetWorldPosition())
end

local function OnSpawn(inst, pet)
    if not inst.nospawnfx then
        --Delayed in case we need to relocate for migration spawning
        pet:DoTaskInTime(0, OnSpawnFX)
    end
    inst:ListenForEvent("onremove", inst.OnPetLost, pet)
end

local function OnDespawn(inst, pet)
    SpawnPrefab("small_puff").Transform:SetPosition(pet.Transform:GetWorldPosition())
    inst:RemoveEventCallback("onremove", inst.OnPetLost, pet)
    pet:Remove()
end

local function OnPetLost(inst, pet)
    local remains = SpawnPrefab("ash")
    local owner = inst.components.inventoryitem.owner
    local holder = owner ~= nil and (owner.components.inventory or owner.components.container) or nil
    if holder ~= nil then
        local slot = holder:GetItemSlot(inst)
        inst:Remove()
        holder:GiveItem(remains, slot)
    else
        StartSpawn(inst)
    end
end
--------------------------------------------------------------
local function RebuildTile(inst)
    if inst.components.inventoryitem:IsHeld() then
        local owner = inst.components.inventoryitem.owner
        inst.components.inventoryitem:RemoveFromOwner(true)
        if owner.components.container then
            owner.components.container:GiveItem(inst)
        elseif owner.components.inventory then
            owner.components.inventory:GiveItem(inst)
        end
    end
end
--------------------------------------------------------------
local function MorphUp_Soul(inst)
    inst._SoulState = "UP"
    RebuildTile(inst)
	
    glutto = glutto or TheSim:FindFirstEntityWithTag("glutto")
    if glutto then
        if glutto.components.follower.leader ~= inst then
            glutto.components.follower:SetLeader(inst)
        end
	end
end
local function MorphNormal_Soul(inst)
    inst._SoulState = "NOM"
    RebuildTile(inst)
end
--------------------------------------------------------------
local function GetSpawnPoint(pt)
    local theta = math.random() * 2 * PI
    local radius = SPAWN_DIST
    local offset = FindWalkableOffset(pt, theta, radius, 12, true)
    return offset ~= nil and (pt + offset) or nil
end

local function SpawnGlutto(inst)
	if not inst.owner then
        local owner = inst.components.inventoryitem.owner
		print("Error: Eyebone has no linked player!")
		--return
	end
	local pt = inst:GetPosition()

    local spawn_pt = GetSpawnPoint(pt)
    if spawn_pt ~= nil then
        local glutto = SpawnPrefab("glutto")
        if glutto ~= nil then
            glutto.Physics:Teleport(spawn_pt:Get())
            glutto:FacePoint(pt:Get())
			if inst.owner then
				inst.owner.glutto_soul = inst
				inst.owner.glutto = glutto
			end

            return glutto
        end
    end
end

local StartRespawn

local function StopRespawn(inst)
    if inst.respawntask ~= nil then
        inst.respawntask:Cancel()
        inst.respawntask = nil
        inst.respawntime = nil
    end
end

local function RebindGlutto(inst, glutto)
	glutto = glutto or (inst.owner and inst.owner.glutto) or TheSim:FindFirstEntityWithTag("glutto")
    if glutto ~= nil then
		if inst.owner then
			inst.components.named:SetName(inst.owner.name.."'s Glutto Soul")
			glutto.components.named:SetName(inst.owner.name.."'s Glutto")
			if inst.ownership then
				glutto:AddTag(ownershiptag)
				glutto:AddTag("uid_" .. inst.owner.userid)
			end
		end
        inst.AnimState:PlayAnimation("idle", true)
        inst:ListenForEvent("death", function() inst:OnGluttoDeath() end, glutto)
        if glutto.components.follower.leader ~= inst then
            glutto.components.follower:SetLeader(inst)
        end
        return true
    end
end

local function RespawnGlutto(inst)
	StopRespawn(inst)
    --RebindGlutto(inst, TheSim:FindFirstEntityWithTag("glutto") or SpawnGlutto(inst))
    RebindGlutto(inst, (inst.owner and inst.owner.glutto) or SpawnGlutto(inst))
end

StartRespawn = function(inst, time)
    StopRespawn(inst)

    time = time or 0
    inst.respawntask = inst:DoTaskInTime(time, RespawnGlutto)
    inst.respawntime = GetTime() + time
    inst.AnimState:PlayAnimation("dead", true)
end

local function OnGluttoDeath(inst)
	if inst.owner then
		inst.owner.glutto = nil
	end
    StartRespawn(inst, TUNING.CHESTER_RESPAWN_TIME)
end

local function FixGlutto(inst)
    inst.fixtask = nil
    --take an existing glutto if there is one
    if not RebindGlutto(inst) then
        inst.AnimState:PlayAnimation("dead", true)
        --inst.components.inventoryitem:ChangeImageName(inst.closedEye)
        
        if inst.components.inventoryitem.owner ~= nil then
            local time_remaining = inst.respawntime ~= nil and math.max(0, inst.respawntime - GetTime()) or 0
            StartRespawn(inst, time_remaining)
        end
    end
end

local function OnPutInInventory(inst)
    if inst.fixtask == nil then
        inst.fixtask = inst:DoTaskInTime(1, FixGlutto)
    end
end

local function OnSound(inst)
	inst.SoundEmitter:PlaySound("millenniumfox/creatures/glutto/_soul")
end

local function OnSave(inst, data)
    data._SoulState = inst._SoulState
    if inst.respawntime ~= nil then
        local time = GetTime()
        if inst.respawntime > time then
            data.respawntimeremaining = inst.respawntime - time
        end
    end
end

local function OnLoad(inst, data)
    if data and data._SoulState then
        if data._SoulState == "UP" then
            inst:MorphUp_Soul()
        end
    end
    if data and data.respawntimeremaining then
		inst.respawntime = data.respawntimeremaining + GetTime()
	end
end

local function GetStatus(inst)
    return inst.respawntask ~= nil and "WAITING" or nil
end

local function fn(Sim)

    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddNetwork()
	
    MakeInventoryPhysics(inst)
	
    inst.entity:AddMiniMapEntity()
    inst.MiniMapEntity:SetIcon("glutto_soul.tex")
	

	inst.entity:AddSoundEmitter()

    inst:AddTag("glutto_soul")
    inst:AddTag("irreplaceable")
	inst:AddTag("nonpotatable")
	
    inst:AddTag("_named")
	
    inst.entity:AddDynamicShadow()
    inst.DynamicShadow:SetSize( 1, .5 )
    
    inst.AnimState:SetBank("glutto_soul")
    inst.AnimState:SetBuild("glutto_soul")
    inst.AnimState:PlayAnimation("idle", true)

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end
	
	inst._SoulState = "NOM"

    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem:SetOnDroppedFn(OnDropped)
	inst.components.inventoryitem:SetOnPutInInventoryFn(OnSound)
    inst.components.inventoryitem.atlasname = "images/inventoryimages/glutto_soul.xml"
	
    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = GetStatus
	inst.components.inspectable:RecordViews()

    inst:AddComponent("leader")
	
    inst:AddComponent("named")
	
    inst:AddComponent("petleash")
    inst.components.petleash:SetPetPrefab("glutto")
    inst.components.petleash:SetOnSpawnFn(OnSpawn)
    inst.components.petleash:SetOnDespawnFn(OnDespawn)
	
	inst.nospawnfx = nil
	
    MakeHauntableLaunch(inst)
	

	--inst.MorphNormal_Soul = MorphNormal_Soul
    --inst.MorphUp_Soul = MorphUp_Soul

    --inst.OnLoad = OnLoad
    --inst.OnSave = OnSave
    --inst.OnGluttoDeath = OnGluttoDeath

    --inst.fixtask = inst:DoTaskInTime(1, FixGlutto)
	--inst.RebindGlutto = RebindGlutto
	
	
    return inst
end

return Prefab( "common/inventory/glutto_soul", fn, assets)