local MakePlayerCharacter = require "prefabs/player_common"

local assets = 
{    
	Asset("SCRIPT", "scripts/prefabs/player_common.lua"),	
	Asset("ANIM", "anim/rash_beard.zip"),	
}
local prefabs = 
{
    "beardhair",
}

-- Custom starting inventory
local start_inv =
{
	"slasher",
	"glutto_soul",
}

-- When the character is revived from human
local function onbecamehuman(inst)
	-- Set speed when not a ghost (optional)
	inst.components.locomotor:SetExternalSpeedMultiplier(inst, "rash_speed_mod", 1)
end

local function onbecameghost(inst)
	-- Remove speed modifier when becoming a ghost
   inst.components.locomotor:RemoveExternalSpeedMultiplier(inst, "rash_speed_mod")
end

-- When loading or spawning the character
local function onload(inst)
    inst:ListenForEvent("ms_respawnedfromghost", onbecamehuman)
    inst:ListenForEvent("ms_becameghost", onbecameghost)

    if inst:HasTag("playerghost") then
        onbecameghost(inst)
    else
        onbecamehuman(inst)
    end
end


-- This initializes for both the server and client. Tags can be added here.
local common_postinit = function(inst) 
	-- Minimap icon
	inst.MiniMapEntity:SetIcon( "rash.tex" )
	
	inst:AddComponent("sanityaura")
    inst.components.sanityaura.aura = -7.5 / 54
	
	inst.soundsname = "rash"
	
	inst:AddTag("rash")	
end

local function OnResetBeard(inst)
    inst.AnimState:ClearOverrideSymbol("beard")
	inst.AnimState:ShowSymbol("hair")
	inst.AnimState:ShowSymbol("hair_hat")
	
    if TUNING.OP == 1 then
        inst.components.locomotor.walkspeed = 3 * TUNING.WILSON_WALK_SPEED
        inst.components.locomotor.runspeed = 3 * TUNING.WILSON_RUN_SPEED
		
    elseif TUNING.OP == 2 then
        inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.RASH_SPEED
        --inst.components.locomotor.runspeed = 1.7 * TUNING.WILSON_RUN_SPEED
		inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.RASH_SPEED
		
    end
end

--tune the beard economy...
local BEARD_DAYS = { 4, 8, 16 }
local BEARD_BITS = { 1, 3,  9 }

local function OnGrowShortBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "rash_beard", "beard_short")
	inst.AnimState:HideSymbol("hair")
	inst.AnimState:HideSymbol("hair_hat")
    inst.components.beard.bits = BEARD_BITS[1]
	inst.components.locomotor.runspeed = 2.5 * TUNING.WILSON_RUN_SPEED
end

local function OnGrowMediumBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "rash_beard", "beard_medium")
	inst.AnimState:HideSymbol("hair")
	inst.AnimState:HideSymbol("hair_hat")
    inst.components.beard.bits = BEARD_BITS[2]
	inst.components.locomotor.runspeed = 4.0 * TUNING.WILSON_RUN_SPEED
end

local function OnGrowLongBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "rash_beard", "beard_long")
	inst.AnimState:HideSymbol("hair")
	inst.AnimState:HideSymbol("hair_hat")
    inst.components.beard.bits = BEARD_BITS[3]
	inst.components.locomotor.runspeed = 5.0 * TUNING.WILSON_RUN_SPEED
end

-- This initializes for the server only. Components are added here.

local master_postinit = function(inst)

	-- choose which sounds this character will play
	--inst.components.locomotor.runspeed = (TUNING.RASH_SPEED)
	
	if TUNING.CHARACTER_SOUND == true
	then
	inst.soundsname = "rash"	
	
	if TUNING.CHARACTER_SOUND == false
	then
	end
	end
	
	inst:AddComponent("beard")
    inst.components.beard.onreset = OnResetBeard
    inst.components.beard.prize ="beardhair"
    inst.components.beard:AddCallback(BEARD_DAYS[1], OnGrowShortBeard)
    inst.components.beard:AddCallback(BEARD_DAYS[2], OnGrowMediumBeard)
    inst.components.beard:AddCallback(BEARD_DAYS[3], OnGrowLongBeard)
		
	-- Uncomment if "wathgrithr"(Wigfrid) or "webber" voice is used
    --inst.talker_path_override = "dontstarve_DLC001/characters/"
	
	-- Stats	
	--inst.components.health:SetMaxHealth(180)
	inst.components.hunger:SetMax(78)
	inst.components.sanity:SetMax(87)
	
	--inst.components.health:StartRegen(5)
		
	-- Damage multiplier (optional)
    inst.components.combat.damagemultiplier = 0.1
	
	-- Hunger rate (optional)
	--inst.components.hunger:hungerrate = (TUNING.RASH_HUNGER)

	
	-- Monster
	inst:AddTag("monster")
	inst.components.locomotor.triggerscreep = false
	
	-- Food
	inst.components.eater.strongstomach = true
	
	inst.OnLoad = onload
    inst.OnNewSpawn = onload
	
	if TUNING.RASH_IMMORTAL == true then --op
			inst.components.health:SetMinHealth(1)
	elseif TUNING.RASH_IMMORTAL == false then -- not op
			inst.components.health:SetMinHealth(0)
	end
	
	if TUNING.OP == 1 then --op
		inst.components.health:SetMaxHealth(780)
		inst.components.hunger:SetMax(100)
		inst.components.sanity:SetMax(100)
		inst.components.combat.damagemultiplier = 7
		inst.components.locomotor.walkspeed = 3.0 * TUNING.WILSON_WALK_SPEED
		inst.components.locomotor.runspeed = 3.0 * TUNING.WILSON_RUN_SPEED
		inst.components.hunger.hungerrate = 0.5 * TUNING.WILSON_HUNGER_RATE
		inst:AddComponent("sanityaura")
		inst.components.sanityaura.aura = -7.5 / 54

	elseif TUNING.OP == 2 then -- not op
		inst.components.health:SetMaxHealth(TUNING.RASH_HEALTH)
		--inst.components.health:SetMaxHealth(380)
		inst.components.hunger:SetMax(78)
		inst.components.sanity:SetMax(87)
		inst.components.combat.damagemultiplier = 3.7
		inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.RASH_SPEED
		inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.RASH_SPEED
		--inst.components.locomotor.walkspeed = 1.7 * TUNING.WILSON_WALK_SPEED
		--inst.components.locomotor.runspeed = 1.7 * TUNING.WILSON_RUN_SPEED
	    inst.components.hunger.hungerrate = TUNING.WILSON_HUNGER_RATE * TUNING.RASH_HUNGER
		--inst.components.hunger.hungerrate = 3.7 * TUNING.WILSON_HUNGER_RATE
		inst:AddComponent("sanityaura")
		inst.components.sanityaura.aura = -2.5 / 54
		
	end
	
end

return MakePlayerCharacter("rash", prefabs, assets, common_postinit, master_postinit, start_inv)